@@ -1,4 +1,4 @@
-# Copyright (c) 2014  Timm Murray
+# Copyright (c) 2015  Timm Murray
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without 
@@ -22,13 +22,14 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 # POSSIBILITY OF SUCH DAMAGE.
 package Device::WebIO::RaspberryPi;
-$Device::WebIO::RaspberryPi::VERSION = '0.008';
+$Device::WebIO::RaspberryPi::VERSION = '0.009';
 # ABSTRACT: Device::WebIO implementation for the Rapsberry Pi
 use v5.12;
 use Moo;
 use namespace::clean;
 use HiPi::Wiring qw( :wiring );
 use HiPi::Device::I2C;
+use HiPi::BCM2835::I2C qw( :all );
 use HiPi::Device::SPI qw( :spi );
 use GStreamer1;
 use Glib qw( TRUE FALSE );
@@ -421,36 +422,38 @@ sub i2c_channels { 2 }
 sub i2c_read
 {
     my ($self, $channel, $addr, $register, $len) = @_;
-    my $dev = $self->_get_i2c_device_by_channel( $channel );
-    my $hipi = HiPi::Device::I2C->new(
-        devicename => $dev,
-        address    => $addr,
-    );
-
-    my @data = $hipi->bus_read( $register, $len );
+    my $dev = $self->_get_i2c_device( $channel, $addr );
+    my @data = $dev->bus_read( $register, $len );
     return @data;
 }
 
 sub i2c_write
 {
     my ($self, $channel, $addr, $register, @data) = @_;
-    my $dev = $self->_get_i2c_device_by_channel( $channel );
-    my $hipi = HiPi::Device::I2C->new(
-        devicename => $dev,
-        address    => $addr,
-    );
-
-    $hipi->bus_write( $register, @data );
+    my $dev = $self->_get_i2c_device( $channel, $addr );
+    $dev->bus_write( $register, @data );
     return 1;
 }
 
-sub _get_i2c_device_by_channel
 {
-    my ($self, $channel) = @_;
-    return
-        $channel == 0 ? '/dev/i2c-0' :
-        $channel == 1 ? '/dev/i2c-1' :
-        undef;
+    my @DEVS;
+    sub _get_i2c_device
+    {
+        my ($self, $channel, $addr) = @_;
+        return $DEVS[$channel]{$addr} if exists $DEVS[$channel]{$addr};
+
+        my $peri = 
+            $channel == 1 ? BB_I2C_PERI_1 :
+            $channel == 0 ? BB_I2C_PERI_0 :
+            undef;
+        my $hipi = HiPi::BCM2835::I2C->new(
+            peripheral => $peri,
+            address    => $addr,
+        );
+
+        $DEVS[$channel]{$addr} = $hipi;
+        return $hipi;
+    }
 }
 
 
@@ -490,6 +493,14 @@ has 'cv' => (
     is      => 'rw',
     default => sub { AnyEvent->condvar },
 );
+has 'vid_use_audio' => (
+    is      => 'rw',
+    default => sub { 0 },
+);
+has 'vid_audio_input_device' => (
+    is      => 'rw',
+    default => sub { 'hw:1,0' },
+);
 with 'Device::WebIO::Device::VideoOutputCallback';
 
 sub vid_channels
@@ -577,6 +588,8 @@ sub vid_stream_begin_loop
     my $bitrate  = $self->vid_kbps( $channel );
     my $callback = $self->_vid_stream_callbacks->[$channel];
     my $type     = $self->_vid_stream_callback_types->[$channel];
+    my $use_audio = $self->vid_use_audio;
+    my $audio_dev = $self->vid_audio_input_device;
 
 
     $self->_init_gstreamer;
@@ -589,6 +602,7 @@ sub vid_stream_begin_loop
         capsfilter => 'the_proud_lord_said' );
     my $sink    = GStreamer1::ElementFactory::make(
         fakesink => 'that_i_should_bow_so_low' );
+    my $vid_queue = GStreamer1::ElementFactory::make( 'queue' => 'only_a_cat' );
 
     my $muxer = ($type ne 'video/H264')
         ? $self->_get_vid_mux_by_type( $type )
@@ -608,16 +622,42 @@ sub vid_stream_begin_loop
         'handoff' => $self->_get_vid_stream_callback( $pipeline, $cv, $callback )
     );
 
-    my @link = ( $rpi, $h264parse, $capsfilter );
-    push @link, $muxer if defined $muxer;
-    push @link, $sink;
-    $pipeline->add( $_ ) for @link;
-    foreach my $i (0 .. ($#link - 1)) {
-        my $this = $link[$i];
-        my $next = $link[$i+1];
-        $this->link( $next );
+    $pipeline->add( $muxer ) if defined $muxer;
+
+    if( $use_audio && defined $muxer ) {
+        my $audio_src = GStreamer1::ElementFactory::make(
+            'alsasrc' => 'of_a_different_coat' );
+        my $audio_caps = GStreamer1::ElementFactory::make(
+            capsfilter => 'the_only_truth_i_know' );
+        my $mp3enc = GStreamer1::ElementFactory::make(
+            lamemp3enc => 'in_a_coat_of_red' );
+        my $audio_queue = GStreamer1::ElementFactory::make(
+            queue => 'or_a_coat_of_gold' );
+
+        $audio_src->set( 'device' => $audio_dev );
+        $mp3enc->set( 'bitrate' => 256 );
+
+        my $caps = GStreamer1::Caps::Simple->new( 'audio/x-raw',
+            rate     => 'Glib::Int'    => 44100,
+            channels => 'Glib::Int'    => 1,
+            format   => 'Glib::String' => 'S16LE',
+        );
+        $audio_caps->set( caps => $caps );
+
+        $pipeline->add( $_ ) for $audio_src, $audio_caps, $mp3enc, $audio_queue;
+        $audio_src->link(   $audio_caps  );
+        $audio_caps->link(  $mp3enc      );
+        $mp3enc->link(      $audio_queue );
+        $audio_queue->link( $muxer       );
     }
 
+    $pipeline->add( $_ ) for $rpi, $h264parse, $capsfilter, $sink, $vid_queue;
+    $rpi->link( $h264parse );
+    $h264parse->link( $capsfilter );
+    $capsfilter->link( $vid_queue );
+    $vid_queue->link( $muxer );
+    $muxer->link( $sink );
+
     $pipeline->set_state( "playing" );
     $cv->recv;
     $pipeline->set_state( "null" );
@@ -827,10 +867,6 @@ __END__
 
 Access the Raspberry Pi's pins using Device::WebIO.
 
-After registering this with the main Device::WebIO object, you shouldn't need 
-to access anything in the Rpi object.  All access should go through the 
-WebIO object.
-
 =head1 CAMERA PREREQUISITES
 
 If you intended to use the camera-related methods (e.g. C<img_stream()>), 
@@ -839,6 +875,14 @@ download and compile this from:
 
 https://github.com/thaytan/gst-rpicamsrc
 
+=head1 VIDEO WITH AUDIO
+
+As of version 0.009, C<Device::WebIO::RaspberryPi> has jumped into the Talkies 
+era of movies by allowing audio input on video streams. You'll need a USB 
+sound device that's compatible with the Raspberry Pi which can also take a 
+microphone input. The attribute C<vid_audio_input_device> can be used to 
+set the ALSA device for recording.
+
 =head1 IMPLEMENTED ROLES
 
 =over 4
@@ -868,9 +912,22 @@ there's any events you would like to handle in between video frames, get this
 condvar and use it with C<AnyEvent>. You may also use this method as a 
 setter before calling C<vid_stream_begin_loop()>.
 
+=head2 vid_use_audio
+
+When getting the video stream, set this flag to true to get put audio on it.  
+Note that this only applies to outputs that are container formats, like AVI, 
+not raw video outputs, like h.264.
+
+Default if false.
+
+=head2 vid_audio_input_device
+
+If putting an audio stream on the video, this specifies the ALSA device to 
+use for input. Default is 'hw:1,0'.
+
 =head1 LICENSE
 
-Copyright (c) 2014  Timm Murray
+Copyright (c) 2015  Timm Murray
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are 
